/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LadspaFX.cpp,v 1.2 2003/12/11 09:05:07 comix Exp $
 *
 */

 #include "LadspaFX.h"

#include <dirent.h>
#include <dlfcn.h>
#include <vector>
using namespace std;

#define LADSPA_IS_CONTROL_INPUT(x) (LADSPA_IS_PORT_INPUT(x) && LADSPA_IS_PORT_CONTROL(x))
#define LADSPA_IS_AUDIO_INPUT(x) (LADSPA_IS_PORT_INPUT(x) && LADSPA_IS_PORT_AUDIO(x))
#define LADSPA_IS_CONTROL_OUTPUT(x) (LADSPA_IS_PORT_OUTPUT(x) && LADSPA_IS_PORT_CONTROL(x))
#define LADSPA_IS_AUDIO_OUTPUT(x) (LADSPA_IS_PORT_OUTPUT(x) && LADSPA_IS_PORT_AUDIO(x))


LadspaFXInfo::LadspaFXInfo()
{
	infoLog( "INIT" );
	sFilename = "";
	sLabel = "";
	sName = "";
	nICPorts = 0;
	nOCPorts = 0;
	nIAPorts = 0;
	nOAPorts = 0;
}


LadspaFXInfo::~LadspaFXInfo()
{
	infoLog( "DESTROY" );
}


///////////////////


// ctor
LadspaFX::LadspaFX( string sLibraryPath, string sPluginLabel )
{
	infoLog( "INIT" );

	handle = NULL;
	m_nICPorts = 0;
	m_nOCPorts = 0;
	m_nIAPorts = 0;
	m_nOAPorts = 0;


	// carico la libreria
	libraryModule = dlopen( sLibraryPath.c_str(), RTLD_NOW );
	if ( !libraryModule ) {
		cout << "errore in dlopen" << endl;
	}

	LADSPA_Descriptor_Function desc_func;
	if ( ( desc_func = (LADSPA_Descriptor_Function)dlsym( libraryModule, "ladspa_descriptor" ) ) ) {
		for ( uint i = 0; ( d = desc_func( i ) ) != NULL; i++ ) {
			string sName = d->Name;
			string sLabel = d->Label;
			uint sID = d->UniqueID;
			string sMaker = d->Maker;
			string sCopyright = d->Copyright;

			if (sLabel != sPluginLabel) {
				continue;
			}

			cout << "PLUGIN FOUND" << endl;

			cout << "name = " << sName << endl;
			cout << "label = " << sLabel << endl;
			cout << "id = " << sID << endl;
			cout << "maker = " << sMaker << endl;
			cout << "copyright = " << sCopyright << endl;

			for (uint j = 0; j < d->PortCount; j++) {
				LADSPA_PortDescriptor pd = d->PortDescriptors[j];
				if ( LADSPA_IS_PORT_INPUT( pd ) && LADSPA_IS_PORT_CONTROL( pd ) ) {
					m_nICPorts++;
				}
				else if ( LADSPA_IS_PORT_INPUT( pd ) && LADSPA_IS_PORT_AUDIO( pd ) ) {
					m_nIAPorts++;
				}
				else if ( LADSPA_IS_PORT_OUTPUT( pd ) && LADSPA_IS_PORT_CONTROL( pd ) ) {
					m_nOCPorts++;
				}
				else if ( LADSPA_IS_PORT_OUTPUT( pd ) && LADSPA_IS_PORT_AUDIO( pd ) ) {
					m_nOAPorts++;
				}
				else {
					cout << "UNKNOWN port ";
				}
			}

			break;
		}
	}
	else {
		cout << "errore in dlsym" << endl;
	}

	if ( (m_nIAPorts != 2) || (m_nOAPorts != 2) ) {
		cout << "numero di porte sbagliato" << endl;
		cout << "in audio = " << m_nIAPorts << endl;
		cout << "out audio = " << m_nOAPorts << endl;
	}

	handle = d->instantiate(d, (long)44100);
}


// dtor
LadspaFX::~LadspaFX()
{
	// dealloca il plugin
	infoLog( "DESTROY" );

	if (d->deactivate) {
		if ( handle ) {
			cout << "deactivate" << endl;
			d->deactivate( handle );
		}
	}

	if (d->cleanup) {
		if ( handle ) {
			cout << "Cleanup" << endl;
			d->cleanup( handle );
		}
	}

	dlclose( libraryModule );
}



void LadspaFX::connectPorts( float* pIn_L, float* pIn_R, float* pOut_L, float* pOut_R )
{
	infoLog( "[LadspaFX::connectPorts]" );
	m_pIn_L = pIn_L;
	m_pIn_R = pIn_R;
	m_pOut_L = pOut_L;
	m_pOut_R = pOut_R;

	uint nAIConn = 0;
	uint nAOConn = 0;
	for ( uint nPort = 0; nPort < d->PortCount; nPort++) {
		LADSPA_PortDescriptor pd = d->PortDescriptors[ nPort ];
		if ( LADSPA_IS_CONTROL_INPUT( pd ) ) {
			LadspaControlPort* pControl = new LadspaControlPort();
			pControl->sName = d->PortNames[ nPort ];
			pControl->fLowerBound = ( d->PortRangeHints[ nPort ] ).LowerBound;
			pControl->fUpperBound = ( d->PortRangeHints[ nPort ] ).UpperBound;
			pControl->fControlValue = pControl->fUpperBound / 2.0;

			inputControlPorts.push_back( pControl );
			d->connect_port( handle, nPort, &(pControl->fControlValue) );
			cout << "\tControl input - " << pControl->sName << endl;
			cout << "\t\tlower bound = " << pControl->fLowerBound << endl;
			cout << "\t\tupper bound = " << pControl->fUpperBound << endl;
		}
		else if ( LADSPA_IS_CONTROL_OUTPUT( pd ) ) {
			LadspaControlPort* pControl = new LadspaControlPort();
			pControl->sName = d->PortNames[ nPort ];
			pControl->fLowerBound = ( d->PortRangeHints[ nPort ] ).LowerBound;
			pControl->fUpperBound = ( d->PortRangeHints[ nPort ] ).UpperBound;
			pControl->fControlValue = pControl->fUpperBound / 2.0;

			outputControlPorts.push_back( pControl );
			d->connect_port( handle, nPort, &(pControl->fControlValue) );
			cout << "\tControl input - " << pControl->sName << endl;
			cout << "\t\tlower bound = " << pControl->fLowerBound << endl;
			cout << "\t\tupper bound = " << pControl->fUpperBound << endl;
		}
		else if ( LADSPA_IS_AUDIO_INPUT( pd ) ) {
			if (nAIConn == 0) {
				d->connect_port( handle, nPort, pIn_L );
				cout << d->PortNames[ nPort ] << endl;
			}
			else if (nAIConn == 1) {
				d->connect_port( handle, nPort, pIn_R );
				cout << d->PortNames[ nPort ] << endl;
			}
			else {
				cout << "troppe porte audio in" << endl;
			}
			nAIConn++;
		}
		else if ( LADSPA_IS_AUDIO_OUTPUT( pd ) ) {
			if (nAOConn == 0) {
				d->connect_port( handle, nPort, pOut_L );
				cout << d->PortNames[ nPort ] << endl;
			}
			else if (nAOConn == 1) {
				d->connect_port( handle, nPort, pOut_R );
				cout << d->PortNames[ nPort ] << endl;
			}
			else {
				cout << "troppe porte audio out" << endl;
			}
		}
		else {
			cout << "unknown port" << endl;
		}
	}


}



void LadspaFX::processFX( uint nFrames )
{
//	infoLog( "[LadspaFX::applyFX()]" );
	d->run( handle, nFrames );
}

void LadspaFX::activate()
{
	infoLog( "[LadspaFX::activate]" );
	if (d->activate) {
		cout << "activate" << endl;
		d->activate( handle );
	}

}


void LadspaFX::deactivate()
{
	infoLog( "[LadspaFX::deactivate]" );
	if (d->deactivate) {
		cout << "deactivate" << endl;
		d->deactivate( handle );
	}
}



///
/// carica solo i plugin validi
///
vector<LadspaFXInfo*> LadspaFX::getPluginList()
{
	cout << "[LadspaFX::getPluginList()]" << endl;

	string sPluginDir = "/usr/lib/ladspa";
	DIR * dir;
	struct dirent * dirent;
	vector<LadspaFXInfo*> pluginVect;

	dir = opendir( sPluginDir.c_str() );
	if ( !dir ) {
		cerr << "directory " << sPluginDir << "not found" << endl;
		return pluginVect;
	}

	while ( (dirent = readdir( dir ) ) != NULL ) {
		string sPluginName = dirent->d_name;
		if ( (sPluginName == ".") || (sPluginName == ".." ) ) {
			continue;
		}
		string sAbsPath = string( sPluginDir ) + string("/") + sPluginName;

		void * module = dlopen( sAbsPath.c_str(), RTLD_NOW );
		if ( !module ) {
			cout << "errore in dlopen" << endl;
		}

		const LADSPA_Descriptor * d;
		LADSPA_Descriptor_Function desc_func;
		if ( ( desc_func = (LADSPA_Descriptor_Function)dlsym( module, "ladspa_descriptor" ) ) ) {
			for ( uint i = 0; (d = desc_func (i)) != NULL; i++) {
				LadspaFXInfo* pFX = new LadspaFXInfo();
				pFX->sFilename = sAbsPath;
				pFX->sLabel = d->Label;
				pFX->sName = d->Name;

				for (uint j = 0; j < d->PortCount; j++) {
					LADSPA_PortDescriptor pd = d->PortDescriptors[j];
					if ( LADSPA_IS_PORT_INPUT( pd ) && LADSPA_IS_PORT_CONTROL( pd ) ) {
						pFX->nICPorts++;
					}
					else if ( LADSPA_IS_PORT_INPUT( pd ) && LADSPA_IS_PORT_AUDIO( pd ) ) {
						pFX->nIAPorts++;
					}
					else if ( LADSPA_IS_PORT_OUTPUT( pd ) && LADSPA_IS_PORT_CONTROL( pd ) ) {
						pFX->nOCPorts++;
					}
					else if ( LADSPA_IS_PORT_OUTPUT( pd ) && LADSPA_IS_PORT_AUDIO( pd ) ) {
						pFX->nOAPorts++;
					}
					else {
						cout << "UNKNOWN port ";
					}
				}
				if ( ( pFX->nIAPorts == 2 ) && ( pFX->nOAPorts == 2 ) ) {
					pluginVect.push_back( pFX );
				}
				else {
					delete pFX;
				}
			}
		}
		dlclose( module );

	}
	return pluginVect;
}

