/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LadspaFX.h,v 1.2 2003/12/11 09:05:07 comix Exp $
 *
 */
#ifndef LADSPA_FX_H
#define LADSPA_FX_H

#include <vector>
using namespace std;
#include "ladspa.h"
#include "Object.h"


class LadspaFXInfo : public Object
{
	public:
		virtual string getClassName() {	return "LadspaFxInfo";	}

		LadspaFXInfo();
		~LadspaFXInfo();

		/// plugin filename
		string sFilename;

		string sLabel;

		string sName;

		/// input control port
		uint nICPorts;

		/// output control port
		uint nOCPorts;

		/// input audio port
		uint nIAPorts;

		/// output audio port
		uint nOAPorts;
};



class LadspaControlPort
{
	public:
		string sName;
		LADSPA_Data fControlValue;
		LADSPA_Data fLowerBound;
		LADSPA_Data fUpperBound;
};



class LadspaFX : public Object
{
	public:
		virtual string getClassName() {	return "LadspaFX";	}

		LadspaFX( string sLibraryPath, string sPluginLabel );
		~LadspaFX();

		void connectPorts( float* pIn_L, float* pIn_R, float* pOut_L, float* pOut_R );
		void activate();
		void deactivate();
		void processFX( uint nFrames );

		static vector<LadspaFXInfo*> getPluginList();

	private:
		void* libraryModule;
		const LADSPA_Descriptor * d;
		LADSPA_Handle handle;

		vector<LadspaControlPort*> inputControlPorts;
		vector<LadspaControlPort*> outputControlPorts;

		uint m_nICPorts;	/// input control port
		uint m_nOCPorts;	/// output control port
		uint m_nIAPorts;	/// input audio port
		uint m_nOAPorts;	/// output audio port

		float* m_pIn_L;
		float* m_pIn_R;
		float* m_pOut_L;
		float* m_pOut_R;
};


#endif


