/*
 * Hydrogen
 * Copyright(c) 2002-2003 by Alex >Comix< Cominu [comix@users.sourceforge.net]
 *
 * http://hydrogen.sourceforge.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY, without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: LocalFileMng.cpp,v 1.1 2003/12/09 11:21:44 comix Exp $
 *
 */

#include "LocalFileMng.h"

#include "tinyxml.h"
#include "../config.h"

//-----------------------------------------------------------------------------
/**
 * Costruttore.
 */
LocalFileMng::LocalFileMng(){
//	infoLog("INIT");
}




//-----------------------------------------------------------------------------
/**
 * Distruttore.
 */
LocalFileMng::~LocalFileMng(){
//	infoLog("DESTROY");
}




/**
 *
 */
vector<string> LocalFileMng::listUserDrumkits() {
	vector<string> list;

	string directory = (PreferencesMng::getInstance())->getUserHome() + "/.hydrogen/data";
	DIR* dirp = opendir( directory.c_str() );
	if( dirp == NULL ) {
		errorLog( "Directory " + string(directory) + " not found" );
	}
	else{
		while( true ) {
			dirent *dirInfo = readdir( dirp );
			if( dirInfo == NULL ){
				break;
			}
			if ( ( dirInfo->d_type != DT_DIR ) && ( dirInfo->d_type != DT_UNKNOWN ) ) {
				warningLog( "listUserDrumkits: file " + directory + "/" + string(dirInfo->d_name) + " is not a directory" );
				continue;
			}

			string filename( dirInfo->d_name );
			if( ( filename == "." ) || ( filename == ".." ) ){
				continue;
			}
			list.push_back( filename );
		}
		closedir( dirp );
	}
	return list;

}




/**
 *
 */
vector<string> LocalFileMng::listSystemDrumkits() {
	vector<string> list;

	string directory = string( DATA_DIR ) + "/drumkits";
	DIR* dirp = opendir( directory.c_str() );
	if( dirp == NULL ) {
		errorLog( "Directory " + string(directory) + " not found" );
	}
	else{
		while( true ) {
			dirent *dirInfo = readdir( dirp );
			if( dirInfo == NULL ){
				break;
			}
			if ( ( dirInfo->d_type != DT_DIR ) && ( dirInfo->d_type != DT_UNKNOWN ) ) {
				warningLog( "listSystemDrumkits: file " + directory + "/" + string(dirInfo->d_name) + " is not a directory" );
				continue;
			}
			string filename( dirInfo->d_name );
			if( ( filename == "." ) || ( filename == ".." ) ){
				continue;
			}
			list.push_back( filename );
		}
		closedir( dirp );
	}
	return list;

}




/**
 *
 */
string LocalFileMng::getDrumkitDirectory( string drumkitName ) {
	// search in system drumkit
	vector<string> systemDrumkits = listSystemDrumkits();
	for ( uint i = 0; i < systemDrumkits.size(); i++ ) {
		if ( systemDrumkits[ i ] == drumkitName ) {
			string path = string( DATA_DIR ) + "/drumkits/";
			return path;
		}
	}

	// search in user drumkit
	vector<string> userDrumkits = listUserDrumkits();
	for ( uint i = 0; i < userDrumkits.size(); i++ ) {
		if ( userDrumkits[ i ] == drumkitName ) {
			string path = (PreferencesMng::getInstance())->getUserHome() + "/.hydrogen/data/";
			return path;
		}
	}

	return "";	// FIXME
}



/**
 *
 */
DrumkitInfo* LocalFileMng::loadDrumkit( string directory ) {
	infoLog( "loadDrumkit " + directory );

	// che if the drumkit.xml file exists
	string drumkitInfoFile = directory + "/drumkit.xml";
	std::ifstream verify( drumkitInfoFile.c_str() , std::ios::in | std::ios::binary );
	if (verify == NULL){
		errorLog( "Load Instrument: Data file " + drumkitInfoFile + " not found." );
		return NULL;
	}

	TiXmlDocument doc( drumkitInfoFile.c_str() );
	doc.LoadFile();

	// root element
	TiXmlNode* drumkitNode;	// root element
	if ( !( drumkitNode = doc.FirstChild( "drumkit_info" ) ) ) {
		errorLog( "Error reading drumkit: drumkit_info node not found" );
		return NULL;
	}

	// Name
	string name = readXmlString( this, drumkitNode, "name", "");
	if (name == "") {
		errorLog( "Error reading drumkit: name node not found" );
		return NULL;
	}

	string author = readXmlString( this, drumkitNode, "author", "undefined author" );
	string info = readXmlString( this, drumkitNode, "info", "defaultInfo" );

	DrumkitInfo *drumkitInfo = new DrumkitInfo();
	drumkitInfo->setName( name );
	drumkitInfo->setAuthor( author );
	drumkitInfo->setInfo( info );

	InstrumentList *instrumentList = new InstrumentList();

	TiXmlNode* instrumentListNode;
	if ( (instrumentListNode = drumkitNode->FirstChild( "instrumentList" ) ) ) {
		// INSTRUMENT NODE
		int instrumentList_count = 0;
		TiXmlNode* instrumentNode = 0;
		for( instrumentNode = instrumentListNode->FirstChild("instrument"); instrumentNode; instrumentNode = instrumentNode->NextSibling("instrument")) {
			instrumentList_count++;

			string id = readXmlString( this, instrumentNode, "id", "" );
			string filename = readXmlString( this, instrumentNode, "filename", "" );
			string name = readXmlString( this, instrumentNode, "name", "" );
			float volume = atof( (readXmlString( this, instrumentNode, "volume", "1.0" )).c_str() );
			bool isMuted = readXmlBool( this, instrumentNode, "isMuted", false );
			float pan_L = atof( (readXmlString( this, instrumentNode, "pan_L", "1.0" )).c_str() );
			float pan_R = atof( (readXmlString( this, instrumentNode, "pan_R", "1.0" )).c_str() );

			Instrument *instr = new Instrument( id, name, "not used", 0, volume );

			// exclude-notes vector
			TiXmlNode* excludeNode = instrumentNode->FirstChild( "exclude" );
			if ( excludeNode ) {
				TiXmlNode* idNode = 0;
				for( idNode = excludeNode->FirstChild( "id"); idNode; idNode = idNode->NextSibling( "id" )) {
					int id = atoi( idNode->FirstChild()->Value() );
					instr->excludeVectId.push_back( id );
				}
			}
			else {
//				warningLog( "Error reading drumkit: exclude node not found" );
			}


			Sample *sample = new Sample( 0, filename );	// used only for filename

			instr->setMuted( isMuted );
			instr->setPan_L( pan_L );
			instr->setPan_R( pan_R );
			instr->setSample( sample );
			instr->setDrumkitName( drumkitInfo->getName() );

			instrumentList->add( instr );
		}
		// prepare the exclude vector
		for (uint nInstr = 0; nInstr < instrumentList->getSize(); nInstr++) {
			Instrument* pInstr = instrumentList->get( nInstr );
			for (uint i = 0; i < pInstr->excludeVectId.size(); i++) {
				int id = pInstr->excludeVectId[ i ];
				Instrument* pExcluded = instrumentList->get( id );
				pInstr->excludeVect.push_back( pExcluded );
			}
		}

	}
	else {
		warningLog( "Error reading drumkit: instrumentList node not found" );
	}
	drumkitInfo->setInstrumentList( instrumentList );

	return drumkitInfo;
}





/**
 *
 */
int LocalFileMng::saveDrumkit( DrumkitInfo *info ) {
	infoLog( "Saving drumkit " + info->getName() );

	// Debug
	info->dump();

	// crete the drumkit directory
	string drumkitDir = ( PreferencesMng::getInstance() )->getUserHome() + string( "/.hydrogen/data/" ) + info->getName();
	mkdir( drumkitDir.c_str(), S_IRWXU );

	// clear all old files in the directory
	string clearCmd = "rm -f " + drumkitDir + "/*";
	system( clearCmd.c_str() );

	// create the drumkit.xml file
	string drumkitXmlFilename = drumkitDir + string( "/drumkit.xml" );

	TiXmlDocument doc( drumkitXmlFilename.c_str() );

	TiXmlElement rootNode( "drumkit_info" );

	writeXmlText( &rootNode, "name", info->getName() );		// name
	writeXmlText( &rootNode, "author", info->getAuthor() );	// author
	writeXmlText( &rootNode, "info", info->getInfo() );		// info

	TiXmlElement instrumentListNode( "instrumentList" );		// instrument list
	uint nInstrument = info->getInstrumentList()->getSize();
	// INSTRUMENT NODE
	for (uint i = 0; i < nInstrument; i++) {
		Instrument *instr = info->getInstrumentList()->get(i);

		// ricavo il nome senza path
		string filename = instr->getSample()->getFilename();
		int index = 0;

		index = filename.rfind( '/' );

		string test = filename;
		filename = filename.substr( index + 1, test.size() - index + 1 );

		string absFilename = drumkitDir + "/" + filename;
		instr->save( absFilename );

		TiXmlElement instrumentNode( "instrument" );

		// id
		writeXmlText( &instrumentNode, "id", instr->getId() );

		// filename
		writeXmlText( &instrumentNode, "filename", filename );

		// name
		writeXmlText( &instrumentNode, "name", instr->getName() );

		// volume
		writeXmlText( &instrumentNode, "volume", instr->getVolume() );

		// is muted?
		string value = "false";
		if (instr->isMuted()) {
			value = "true";
		}
		writeXmlText( &instrumentNode, "isMuted", value );

		// pan L
		writeXmlText( &instrumentNode, "pan_L", instr->getPan_L() );

		// pan R
		writeXmlText( &instrumentNode, "pan_R", instr->getPan_R() );

		// exclude vector
		TiXmlElement excludeNode( "exclude" );
		if (instr->excludeVectId.size() != 0) {
			for (uint i = 0; i <instr->excludeVectId.size(); i++) {
				writeXmlText( &excludeNode, "id" , instr->excludeVectId[ i ] );
			}
		}
		instrumentNode.InsertEndChild( excludeNode);

		instrumentListNode.InsertEndChild(instrumentNode);
	}

	rootNode.InsertEndChild(instrumentListNode);

	doc.InsertEndChild( rootNode );
	doc.SaveFile();

	return 0; // ok
}





/**
 *
 */
void LocalFileMng::installDrumkit( string filename ) {
	infoLog( "install drumkit " + filename );

	string dataDir = ( PreferencesMng::getInstance() )->getUserHome() + string( "/.hydrogen/data/" );

	// unpack the drumkit
	string cmd = string( "cd " ) + dataDir + string( "; tar xzf " ) + filename;
	system( cmd.c_str() );
}




/**
 *
 */
int LocalFileMng::uninstallDrumkit( string drumkitName ) {
	infoLog( "uninstall drumkit " + drumkitName );

	// verificare che non sia un drumkit di sistema

	return 0;	// OK
}




//-----------------------------------------------------------------------------
/**
 * Carica la canzone specificata.
 */
Song* LocalFileMng::loadSong(string filename) {

	Song *song = NULL;

	SongReader reader;
	song = reader.readSong(filename);

	return song;
}







//-----------------------------------------------------------------------------
/**
 * Save the song
 */
void LocalFileMng::saveSong(Song *song, string filename) {
	SongWriter writer;

	writer.writeSong(song, filename);
}













//-----------------------------------------------------------------------------
//	Implementation of SongReader class
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/**
 * Constructor
 */
SongReader::SongReader() {
//	infoLog("init");
}





//-----------------------------------------------------------------------------
/**
 * Destructor
 */
SongReader::~SongReader() {
//	infoLog("destroy");
}



//-----------------------------------------------------------------------------
/**
 * Read a song.
 * return NULL = error reading song file
 */
Song* SongReader::readSong(string filename) {
	infoLog("loading song " + filename);
	Song* song = NULL;
	bool convertTo081Format = false;

	// song file exists??
	std::ifstream verify(filename.c_str() , std::ios::in | std::ios::binary);
	if (verify == NULL) {
		errorLog("Song file " + filename + " not found.");
		return NULL;
	}


	TiXmlDocument doc(filename.c_str());
	doc.LoadFile();

	// root element
	TiXmlNode* songNode;	// root element
	if ( !( songNode = doc.FirstChild("song") ) ) {
		errorLog("Error reading song: song node not found");
		return NULL;
	}

	// Version
	TiXmlNode* versionNode;
	if ( ( versionNode = songNode->FirstChild("version") ) ) {
		string version = versionNode->FirstChild()->Value();
		if (version != VERSION) {
			char tmp[200];
			sprintf( tmp, "Trying to load a song created with a different version of hydrogen. Song saved with version %s", version.c_str());
			warningLog( tmp );
			if ( (version == "0.8.0") || (version == "0.7.6") || (version == "0.8.0beta3") || (version == "0.8.0beta2") ) {
				convertTo081Format = true;
			}
		}
	}
	else {
		warningLog("Error reading song: version node not found");
		return NULL;
	}


	// Bpm
	float bpm = 120;
	TiXmlNode* bpmNode;
	if ( (bpmNode = songNode->FirstChild("bpm") ) ) {
		bpm = atof(bpmNode->FirstChild()->Value());
	}
	else {
		warningLog("Error reading song: bpm node not found");
	}

	// Volume
	float volume = 0.5;
	TiXmlNode* volumeNode;
	if ( ( volumeNode = songNode->FirstChild("volume")) ) {
		volume = atof(volumeNode->FirstChild()->Value());
	}
	else {
		warningLog("Error reading song: volume node not found");
	}


	// Metronome Volume
	float metronomeVolume = 0.50f;
	TiXmlNode* metronomeVolumeNode;
	if ( ( metronomeVolumeNode = songNode->FirstChild("metronomeVolume") ) ) {
		metronomeVolume = atof(metronomeVolumeNode->FirstChild()->Value());
	}
	else {
		warningLog("Error reading song: metronomeVolume node not found");
	}

	// Name
	string name = "Untitled";
	TiXmlNode* nameNode;
	if ( (nameNode = songNode->FirstChild("name") ) ) {
		TiXmlNode* nameTxt = nameNode->FirstChild();
		if (nameTxt) {
			name = nameTxt->Value();
		}
	}
	else {
		warningLog("Error reading song: name node not found");
	}


	// Author
	string author = "Author";
	TiXmlNode* authorNode;
	if ( ( authorNode = songNode->FirstChild("author") ) ) {
		TiXmlNode* authorTxt = authorNode->FirstChild();
		if (authorTxt) {
			author = authorTxt->Value();
		}
	}
	else {
		warningLog("Error reading song: author node not found");
	}


	// Notes
	string notes = "";
	TiXmlNode* notesNode;
	if ( (notesNode = songNode->FirstChild("notes") ) ) {
		TiXmlNode *notesTxt = notesNode->FirstChild();
		if (notesTxt) {
			notes = notesTxt->Value();
		}
	}
	else {
		warningLog("Error reading song: notes node not found");
	}


	// Loop enabled
	bool loopEnabled = false;
	string loopEnabledStr = "";
	TiXmlNode* loopEnabledNode;
	if ( (loopEnabledNode = songNode->FirstChild("loopEnabled") ) ) {
		TiXmlNode *loopEnabledTxt = loopEnabledNode->FirstChild();
		if (loopEnabledTxt) {
			loopEnabledStr = loopEnabledTxt->Value();
		}
		if ( loopEnabledStr == "true") {
			loopEnabled = true;
		}
	}
	else {
		warningLog("Error reading song: loopEnabled node not found");
	}

	// Mode (song/live)
	int mode = LIVE_MODE;
	TiXmlNode* modeNode;
	if ( (modeNode = songNode->FirstChild( "mode" ) ) ) {
		TiXmlNode *modeTxt = modeNode->FirstChild();
		if (modeTxt) {
			string modeStr = modeTxt->Value();
			if (modeStr == "live") {
				mode = LIVE_MODE;
			}
			else if ( modeStr == "song" ) {
				mode = SONG_MODE;
			}
		}
	}
	else {
		warningLog("Error reading song: mode node not found");
	}

	// humanize time enabled
	bool humanizeTimeEnabled = false;
	string humanizeTimeEnabledStr = "";
	TiXmlNode* humanizeTimeEnabledNode;
	if ( (humanizeTimeEnabledNode = songNode->FirstChild( "humanizeTimeEnabled" ) ) ) {
		TiXmlNode *humanizeTimeEnabledTxt = humanizeTimeEnabledNode->FirstChild();
		if (humanizeTimeEnabledTxt) {
			humanizeTimeEnabledStr = humanizeTimeEnabledTxt->Value();
		}
		if ( humanizeTimeEnabledStr == "true") {
			humanizeTimeEnabled = true;
		}
	}
	else {
		warningLog( "Error reading song: humanizeTimeEnabled node not found" );
	}

	// humanize time
	float humanizeTimeValue = 0.0;
	TiXmlNode* humanizeTimeNode;
	if ( ( humanizeTimeNode = songNode->FirstChild("humanize_time") ) ) {
		humanizeTimeValue = atof( humanizeTimeNode->FirstChild()->Value() );
	}
	else {
		warningLog("Error reading song: humanize_time node not found");
	}

	// humanize velocity enabled
	bool humanizeVelocityEnabled = false;
	string humanizeVelocityEnabledStr = "";
	TiXmlNode* humanizeVelocityEnabledNode;
	if ( (humanizeVelocityEnabledNode = songNode->FirstChild( "humanizeVelocityEnabled" ) ) ) {
		TiXmlNode *humanizeVelocityEnabledTxt = humanizeVelocityEnabledNode->FirstChild();
		if (humanizeVelocityEnabledTxt) {
			humanizeVelocityEnabledStr = humanizeVelocityEnabledTxt->Value();
		}
		if ( humanizeVelocityEnabledStr == "true") {
			humanizeVelocityEnabled = true;
		}
	}
	else {
		warningLog( "Error reading song: humanizeVelocityEnabled node not found" );
	}

	// humanize velocity
	float humanizeVelocityValue = 0.0;
	TiXmlNode* humanizeVelocityNode;
	if ( ( humanizeVelocityNode = songNode->FirstChild("humanize_velocity") ) ) {
		humanizeVelocityValue = atof( humanizeVelocityNode->FirstChild()->Value() );
	}
	else {
		warningLog("Error reading song: humanize_velocity node not found");
	}

	// swing enabled
	bool swingEnabled = false;
	string swingEnabledStr = "";
	TiXmlNode* swingEnabledNode;
	if ( (swingEnabledNode = songNode->FirstChild( "swingEnabled" ) ) ) {
		TiXmlNode *swingEnabledTxt = swingEnabledNode->FirstChild();
		if (swingEnabledTxt) {
			swingEnabledStr = swingEnabledTxt->Value();
		}
		if ( swingEnabledStr == "true") {
			swingEnabled = true;
		}
	}
	else {
		warningLog( "Error reading song: swingEnabled node not found" );
	}

	// swing factor
	float swingFactor = 0.0;
	TiXmlNode* swingFactorNode;
	if ( ( swingFactorNode = songNode->FirstChild("swing_factor") ) ) {
		swingFactor = atof( swingFactorNode->FirstChild()->Value() );
	}
	else {
		warningLog("Error reading song: swing_factor node not found");
	}


	// delay enabled
	bool delayEnabled = false;
	string delayEnabledStr = "";
	TiXmlNode* delayEnabledNode;
	if ( (delayEnabledNode = songNode->FirstChild("delayEnabled") ) ) {
		TiXmlNode *delayEnabledTxt = delayEnabledNode->FirstChild();
		if (delayEnabledTxt) {
			delayEnabledStr = delayEnabledTxt->Value();
		}
		if ( delayEnabledStr == "true") {
			delayEnabled = true;
		}
	}
	else {
		warningLog("Error reading song: delayEnabled node not found");
	}

	// delay wet level
	float delayWetLevel = 0.0;
	TiXmlNode* delayWetLevelNode;
	if ( ( delayWetLevelNode = songNode->FirstChild( "delay_wet_level" ) ) ) {
		delayWetLevel = atof( delayWetLevelNode->FirstChild()->Value() );
	}
	else {
		warningLog("Error reading song: delay_wet_level node not found");
	}

	// delay time
	int delayTime = 16;
	TiXmlNode* delayTimeNode;
	if ( ( delayTimeNode = songNode->FirstChild( "delay_time" ) ) ) {
		delayTime = atoi( delayTimeNode->FirstChild()->Value() );
	}
	else {
		warningLog("Error reading song: delay_time node not found");
	}

	// delay feedback
	float delayFeedbackLevel = 0.0;
	TiXmlNode* delayFeedbackNode;
	if ( ( delayFeedbackNode = songNode->FirstChild( "delay_feedback_level" ) ) ) {
		delayFeedbackLevel = atof( delayFeedbackNode->FirstChild()->Value() );
	}
	else {
		warningLog("Error reading song: delay_feedback_level node not found");
	}


	song = new Song( "id", name, author, bpm, volume );
	song->setMetronomeVolume( metronomeVolume );
	song->setNotes( notes );
	song->setLoopEnabled( loopEnabled );
	song->setMode( mode );
	song->setHumanizeTimeEnabled( humanizeTimeEnabled );
	song->setHumanizeTimeValue( humanizeTimeValue );
	song->setHumanizeVelocityEnabled( humanizeVelocityEnabled );
	song->setHumanizeVelocityValue( humanizeVelocityValue );
	song->setSwingEnabled( swingEnabled );
	song->setSwingFactor( swingFactor );
	song->setDelayEnabled( delayEnabled );
	song->setDelayWetLevel( delayWetLevel );
	song->setDelayTime( delayTime );
	song->setDelayFeedbackLevel( delayFeedbackLevel );


	//  Instrument List
	LocalFileMng localFileMng;
	InstrumentList *instrumentList = new InstrumentList();

	TiXmlNode* instrumentListNode;
	if ( (instrumentListNode = songNode->FirstChild("instrumentList") ) ) {
		// INSTRUMENT NODE
		int instrumentList_count = 0;
		TiXmlNode* instrumentNode = 0;
		for( instrumentNode = instrumentListNode->FirstChild("instrument"); instrumentNode; instrumentNode = instrumentNode->NextSibling("instrument")) {
			instrumentList_count++;

			TiXmlNode* idNode =  instrumentNode->FirstChild("id");
			string id = idNode->FirstChild()->Value();

			TiXmlNode* drumkitNode =  instrumentNode->FirstChild( "drumkit" );
			string drumkit = "";
			if (drumkitNode) {
				if (drumkitNode->FirstChild()) {
					drumkit = drumkitNode->FirstChild()->Value();
				}
			}
			else {
				warningLog( "Error Loading song: drumkit node not found" );
			}

			TiXmlNode* filenameNode =  instrumentNode->FirstChild("filename");
			string filename = filenameNode->FirstChild()->Value();

			// name
			TiXmlNode* nameNode =  instrumentNode->FirstChild("name");
			string name = nameNode->FirstChild()->Value();

			// volume
			float volume = 1.0;
			TiXmlNode* volumeNode =  instrumentNode->FirstChild("volume");
			volume = atof(volumeNode->FirstChild()->Value());

			// is muted
			TiXmlNode* isMutedNode =  instrumentNode->FirstChild("isMuted");
			string isMutedStr = isMutedNode->FirstChild()->Value();

			// pan L
			float pan_L = 1.0;
			TiXmlNode* pan_L_Node =  instrumentNode->FirstChild("pan_L");
			pan_L = atof(pan_L_Node->FirstChild()->Value());

			// pan R
			float pan_R = 1.0;
			TiXmlNode* pan_R_Node =  instrumentNode->FirstChild("pan_R");
			pan_R = atof(pan_R_Node->FirstChild()->Value());

			// delay level
			float delayLevel = 0.0;
			TiXmlNode* delayLevelNode =  instrumentNode->FirstChild( "delayLevel" );
			if ( delayLevelNode ) {
				delayLevel = atof( delayLevelNode->FirstChild()->Value() );
			}
			else {
				warningLog( "Error Loading song: delayLevel node not found" );
			}

			string drumkitPath = "";
			if ( drumkit != "" ) {
				drumkitPath = localFileMng.getDrumkitDirectory( drumkit ) + drumkit + "/";
			}
			Instrument *instr = Instrument::load( drumkitPath + filename );
			if (instr) { // is instrument valid?
				instr->getSample()->setFilename( filename );
			}
			else {
				errorLog( "Error Loading song: NULL instrument, now using /emptySample.wav" );
				instr = Instrument::load( string(DATA_DIR) + "/emptySample.wav" );
			}

			// exclude-notes vector
			TiXmlNode* excludeNode = instrumentNode->FirstChild( "exclude" );
			if ( excludeNode ) {
				TiXmlNode* idNode = 0;
				for( idNode = excludeNode->FirstChild( "id"); idNode; idNode = idNode->NextSibling( "id" )) {
					int id = atoi( idNode->FirstChild()->Value() );
					instr->excludeVectId.push_back( id );
				}
			}
			else {
				warningLog( "Error loading song: exclude node not found" );
			}


			instr->setId( id );
			instr->setVolume( volume );
			instr->setName( name );
			if ( isMutedStr == "true" ) {
				instr->setMuted( true );
			}
			else {
				instr->setMuted( false );
			}
			instr->setPan_L( pan_L );
			instr->setPan_R( pan_R );
			instr->setDelayLevel( delayLevel );
			if ( drumkit != "" ) {
				instr->setDrumkitName( drumkit );
			}

			instrumentList->add(instr);
		}
		// prepare the exclude vector
		for (uint nInstr = 0; nInstr < instrumentList->getSize(); nInstr++) {
			Instrument* pInstr = instrumentList->get( nInstr );
			if (pInstr) {
				for (uint i = 0; i < pInstr->excludeVectId.size(); i++) {
					int id = pInstr->excludeVectId[ i ];
					Instrument* pExcluded = instrumentList->get( id );
					pInstr->excludeVect.push_back( pExcluded );
				}
			}
			else {
				errorLog( "[SongReader::readSong] pInstr == NULL" );
			}
		}


		song->setInstrumentList( instrumentList );
		if ( instrumentList_count != MAX_TRACKS ) {
			warningLog( "Instrument number != MAX_INSTRUMENTS");

			// create the missing instruments with empty sample
			uint nInstrMissing = MAX_TRACKS - instrumentList_count;
			for (uint i = 0; i < nInstrMissing; i++) {
				int instrId = instrumentList_count + i;
				warningLog( "Creating empty instrument");

				Instrument *instr = Instrument::load( string(DATA_DIR) + "/emptySample.wav" );
				char idStr[10];
				sprintf(idStr,"%d", instrId);
				instr->setId( idStr );
				instr->setVolume( 0.1 );
				instr->setName( "Empty" );
				instr->setMuted( false );
				instr->setPan_L( 1.0 );
				instr->setPan_R( 1.0 );
				instr->setDelayLevel( 0.0 );
				instrumentList->add( instr );
			}
		}
	}
	else {
		errorLog("Error reading song: instrumentList node not found");
		delete song;
		return NULL;
	}





	// Pattern list
	TiXmlNode* patterns = songNode->FirstChild("patternList");

	PatternList *patternList = new PatternList();
	int pattern_count = 0;
	TiXmlNode* patternNode = 0;
	for (patternNode = patterns->FirstChild("pattern"); patternNode; patternNode = patternNode->NextSibling( "pattern" )) {
		pattern_count++;
		Pattern *pat = getPattern(patternNode, instrumentList);
		patternList->add(pat);
	}
	song->setPatternList(patternList);


	// Pattern sequence
	TiXmlNode* patternSequenceNode = songNode->FirstChild("patternSequence");

	PatternList *patternSequence = new PatternList();

	int patternSequence_count = 0;
	TiXmlNode* patternId = 0;
	for (patternId = patternSequenceNode->FirstChild("patternID"); patternId; patternId = patternId->NextSibling("patternID")) {
		patternSequence_count++;
		string patId = patternId->FirstChild()->Value();

		Pattern *pat = NULL;

		for (uint i = 0; i < patternList->getSize(); i++) {
			Pattern *tmp = patternList->get(i);
			if (tmp) {
				if (tmp->getName() == patId) {
					pat = tmp;
					break;
				}
			}
		}
		patternSequence->add( pat );
	}
	song->setPatternSequence(patternSequence);

	song->setModified(false);
	song->setFilename(filename);


	if (convertTo081Format) {
		warningLog("Converting from old song format...");
		for (uint nPattern = 0; nPattern < song->getPatternList()->getSize(); nPattern++) {
			Pattern *pat = song->getPatternList()->get(nPattern);
			pat->setSize( pat->getSize() * 192 / 64 );

			for (uint nSequence = 0; nSequence < MAX_TRACKS; nSequence++) {
				Sequence *sequence = pat->getSequenceList()->get(nSequence);
				for (uint nNote = 0; nNote < MAX_NOTES; nNote++) {
					Note *note = sequence->noteList[nNote];
					if (note) {
						int nPos = note->getPosition() * 192 / 64;
						note->setPosition( nPos );
					}
				}
			}
		}
		warningLog("Save song with new format");
		SongWriter writer;
		writer.writeSong(song, filename);
		delete song;
		return readSong(filename);
	}

	return song;
}



//-----------------------------------------------------------------------------
/**
 *
 */
Pattern* SongReader::getPattern(TiXmlNode* pattern, InstrumentList* instrList){
	Pattern *pat = NULL;

	TiXmlNode* nameNode = pattern->FirstChild("name");
	string name = nameNode->FirstChild()->Value();

	TiXmlNode* pSizeNode = pattern->FirstChild("size");
	int nSize = 64;
	if ( pSizeNode ) {
		nSize = atoi( pSizeNode->FirstChild()->Value() );
	}
	else {
		warningLog( "pattern::size node not found. Using 64 ticks default." );
	}

	pat = new Pattern( name, nSize );

	SequenceList *sequenceList = new SequenceList();

	TiXmlNode* sequenceListNode = pattern->FirstChild("sequenceList");

	int sequence_count = 0;
	TiXmlNode* sequenceNode = 0;
	for (sequenceNode = sequenceListNode->FirstChild("sequence"); sequenceNode; sequenceNode = sequenceNode->NextSibling("sequence")) {
		sequence_count++;
		Sequence *seq = getSequence(sequenceNode, instrList);

		sequenceList->add(seq);
	}
	pat->setSequenceList(sequenceList);
	if ( sequence_count != MAX_TRACKS ) {
		warningLog( "sequence number != MAX_INSTRUMENTS" );

		uint nMissingSequences = MAX_TRACKS - sequence_count;

		for (uint i = 0; i < nMissingSequences; i++) {
			warningLog( "Creating empty Sequence");
			Sequence *seq = new Sequence( "empty seq" );

			sequenceList->add(seq);

		}

	}
	return pat;
}





//-----------------------------------------------------------------------------
/**
 *
 */
Sequence* SongReader::getSequence(TiXmlNode* sequence, InstrumentList* instrList) {
	Sequence* seq = NULL;

//	TiXmlNode* trackNo = sequence->FirstChild("trackNo");

	TiXmlNode* nameNode = sequence->FirstChild("name");
	string name = nameNode->FirstChild()->Value();

	seq = new Sequence(name);

	TiXmlNode* noteListNode = sequence->FirstChild("noteList");

	TiXmlNode* noteNode = 0;
	for (noteNode = noteListNode->FirstChild("note"); noteNode; noteNode = noteNode->NextSibling("note")) {
		Note *note = getNote(noteNode, instrList);
		seq->noteList[note->getPosition()] = note;
	}

	return seq;
}




//-----------------------------------------------------------------------------
/**
 *
 */
Note* SongReader::getNote(TiXmlNode* noteNode, InstrumentList *instrList){
	Note* note = NULL;

	TiXmlNode* positionNode =  noteNode->FirstChild("position");
	uint position = atoi(positionNode->FirstChild()->Value());

	TiXmlNode* velocityNode = noteNode->FirstChild("velocity");
	float velocity = atof(velocityNode->FirstChild()->Value());

	TiXmlNode* pan_LNode = noteNode->FirstChild("pan_L");
	float pan_L = atof(pan_LNode->FirstChild()->Value());

	TiXmlNode* pan_RNode = noteNode->FirstChild("pan_R");
	float pan_R = atof(pan_RNode->FirstChild()->Value());

	float nPitch = 0.0;
	TiXmlNode* pitchNode = noteNode->FirstChild( "pitch" );
	if (pitchNode) {
		nPitch = atof(pitchNode->FirstChild()->Value());
	}

	TiXmlNode* instrNode = noteNode->FirstChild("instrument");
	string instrId = instrNode->FirstChild()->Value();

	Instrument *instrRef = NULL;
	// search instrument by ref
	for (uint i = 0; i < instrList->getSize(); i++) {
		Instrument *instr = instrList->get(i);
		if (instrId == instr->getId()) {
			instrRef = instr;
			break;
		}
	}
	if (instrRef == NULL) {
		errorLog("instrRef NULL");
	}

	note = new Note(position, velocity, pan_L, pan_R, nPitch);
	note->setInstrument(instrRef);

	return note;
}






//-----------------------------------------------------------------------------
//	Implementation of SongWriter class
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
/**
 * Constructor
 */
SongWriter::SongWriter() {
//	infoLog("init");
}





//-----------------------------------------------------------------------------
/**
 * Destructor
 */
SongWriter::~SongWriter() {
//	infoLog("destroy");
}





//-----------------------------------------------------------------------------
/**
 *
 */
void SongWriter::writeSong(Song *song, string filename) {
	infoLog( "Saving song " + filename );

	// TODO: verificare se e' possibile scrivere il file
	// TODO: verificare che il file non sia gia' esistente
	// TODO: effettuare copia di backup per il file gia' esistente

	char temp[255];
	TiXmlDocument doc(filename);

	//song
	TiXmlElement songNode("song");

	// Engine version
	TiXmlElement versionNode("version");
	TiXmlText versionValue(VERSION);
	versionNode.InsertEndChild(versionValue);
	songNode.InsertEndChild(versionNode);

	//bpm
	TiXmlElement bpmNode("bpm");
	sprintf(temp, "%f", song->getBpm());
	TiXmlText bpmValue(temp);
	bpmNode.InsertEndChild(bpmValue);
	songNode.InsertEndChild(bpmNode);

	// Volume
	TiXmlElement volumeNode("volume");
	sprintf(temp, "%f", song->getVolume());
	TiXmlText volumeValue(temp);
	volumeNode.InsertEndChild(volumeValue);
	songNode.InsertEndChild(volumeNode);

	// Metronome volume
	TiXmlElement metronomeVolumeNode("metronomeVolume");
	sprintf(temp, "%f", song->getMetronomeVolume());
	TiXmlText metronomeVolumeValue(temp);
	metronomeVolumeNode.InsertEndChild(metronomeVolumeValue);
	songNode.InsertEndChild(metronomeVolumeNode);

	// Name
	TiXmlElement nameNode("name");
	TiXmlText nameValue(song->getName());
	nameNode.InsertEndChild(nameValue);
	songNode.InsertEndChild(nameNode);

	// Author
	TiXmlElement authorNode("author");
	TiXmlText authorValue(song->getAuthor());
	authorNode.InsertEndChild(authorValue);
	songNode.InsertEndChild(authorNode);

	// Notes
	TiXmlElement notesNode("notes");
	TiXmlText notesValue(song->getNotes());
	notesNode.InsertEndChild(notesValue);
	songNode.InsertEndChild(notesNode);

	// loopEnabled
	TiXmlElement loopEnabledNode("loopEnabled");
	string loopValue = "false";
	if ( song->isLoopEnabled() ) {
		loopValue = "true";
	}
	TiXmlText loopEnabledValue( loopValue );
	loopEnabledNode.InsertEndChild( loopEnabledValue );
	songNode.InsertEndChild( loopEnabledNode );

	// mode (song/live)
	TiXmlElement modeNode( "mode" );
	string modeStr = "live";
	if (song->getMode() == SONG_MODE ) {
		modeStr = "song";
	}
	TiXmlText modeValue( modeStr );
	modeNode.InsertEndChild( modeValue );
	songNode.InsertEndChild( modeNode );

	// humanize time Enabled
	TiXmlElement humanizeTimeEnabledNode( "humanizeTimeEnabled" );
	string humanizeTimeEnabledStr = "false";
	if ( song->isHumanizeTimeEnabled() ) {
		humanizeTimeEnabledStr = "true";
	}
	TiXmlText humanizeTimeEnabledValue( humanizeTimeEnabledStr );
	humanizeTimeEnabledNode.InsertEndChild( humanizeTimeEnabledValue );
	songNode.InsertEndChild( humanizeTimeEnabledNode );

	// Humanize time
	TiXmlElement humanizeTimeNode( "humanize_time" );
	sprintf( temp, "%f", song->getHumanizeTimeValue() );
	TiXmlText humanizeTimeValue( temp );
	humanizeTimeNode.InsertEndChild( humanizeTimeValue );
	songNode.InsertEndChild( humanizeTimeNode );

	// humanize velocity Enabled
	TiXmlElement humanizeVelocityEnabledNode( "humanizeVelocityEnabled" );
	string humanizeVelocityEnabledStr = "false";
	if ( song->isHumanizeVelocityEnabled() ) {
		humanizeVelocityEnabledStr = "true";
	}
	TiXmlText humanizeVelocityEnabledValue( humanizeVelocityEnabledStr );
	humanizeVelocityEnabledNode.InsertEndChild( humanizeVelocityEnabledValue );
	songNode.InsertEndChild( humanizeVelocityEnabledNode );

	// Humanize Velocity
	TiXmlElement humanizeVelocityNode( "humanize_velocity" );
	sprintf( temp, "%f", song->getHumanizeVelocityValue() );
	TiXmlText humanizeVelocityValue( temp );
	humanizeVelocityNode.InsertEndChild( humanizeVelocityValue );
	songNode.InsertEndChild( humanizeVelocityNode );

	// swing Enabled
	TiXmlElement swingEnabledNode( "swingEnabled" );
	string swingEnabledStr = "false";
	if ( song->isSwingEnabled() ) {
		swingEnabledStr = "true";
	}
	TiXmlText swingEnabledValue( swingEnabledStr );
	swingEnabledNode.InsertEndChild( swingEnabledValue );
	songNode.InsertEndChild( swingEnabledNode );

	// Swing factor
	TiXmlElement swingFactorNode( "swing_factor" );
	sprintf( temp, "%f", song->getSwingFactor() );
	TiXmlText swingFactorValue( temp );
	swingFactorNode.InsertEndChild( swingFactorValue );
	songNode.InsertEndChild( swingFactorNode );

	// delay Enabled
	TiXmlElement delayEnabledNode("delayEnabled");
	string delayEnabledStr = "false";
	if ( song->isDelayEnabled() ) {
		delayEnabledStr = "true";
	}
	TiXmlText delayEnabledValue( delayEnabledStr );
	delayEnabledNode.InsertEndChild( delayEnabledValue );
	songNode.InsertEndChild( delayEnabledNode );

	// delay wet level
	TiXmlElement delayWetLevelNode( "delay_wet_level" );
	sprintf( temp, "%f", song->getDelayWetLevel() );
	TiXmlText delayWetLevelValue( temp );
	delayWetLevelNode.InsertEndChild( delayWetLevelValue );
	songNode.InsertEndChild( delayWetLevelNode );

	// delay time
	TiXmlElement delayTimeNode( "delay_time" );
	sprintf( temp, "%d", song->getDelayTime() );
	TiXmlText delayTimeValue( temp );
	delayTimeNode.InsertEndChild( delayTimeValue );
	songNode.InsertEndChild( delayTimeNode );

	// delay feedback level
	TiXmlElement delayFeedbackLevelNode( "delay_feedback_level" );
	sprintf( temp, "%f", song->getDelayFeedbackLevel() );
	TiXmlText delayFeedbackLevelValue( temp );
	delayFeedbackLevelNode.InsertEndChild( delayFeedbackLevelValue );
	songNode.InsertEndChild( delayFeedbackLevelNode );


	// instrument list
	TiXmlElement instrumentListNode("instrumentList");
	uint nInstrument = song->getInstrumentList()->getSize();

	// INSTRUMENT NODE
	for (uint i = 0; i < nInstrument; i++) {
		Instrument *instr = song->getInstrumentList()->get(i);

		TiXmlElement instrumentNode("instrument");


		TiXmlElement instrumentIDNode("id");
		TiXmlText instrumentIDValue(instr->getId());
		instrumentIDNode.InsertEndChild(instrumentIDValue);
		instrumentNode.InsertEndChild(instrumentIDNode);

		TiXmlElement drumkitNode( "drumkit" );
		TiXmlText drumkitValue( instr->getDrumkitName() );
		drumkitNode.InsertEndChild( drumkitValue );
		instrumentNode.InsertEndChild( drumkitNode );

		TiXmlElement filenameNode("filename");
		TiXmlText filenameValue(instr->getSample()->getFilename());
		filenameNode.InsertEndChild(filenameValue);
		instrumentNode.InsertEndChild(filenameNode);

		TiXmlElement nameNode("name");
		TiXmlText nameValue(instr->getName());
		nameNode.InsertEndChild(nameValue);
		instrumentNode.InsertEndChild(nameNode);

		TiXmlElement volumeNode("volume");
		sprintf(temp, "%f", instr->getVolume());
		TiXmlText volumeValue(temp);
		volumeNode.InsertEndChild(volumeValue);
		instrumentNode.InsertEndChild(volumeNode);

		TiXmlElement isMutedNode("isMuted");
		string value = "false";
		if (instr->isMuted()) {
			value = "true";
		}
		TiXmlText isMutedValue(value);
		isMutedNode.InsertEndChild(isMutedValue);
		instrumentNode.InsertEndChild(isMutedNode);


		TiXmlElement pan_LNode("pan_L");
		sprintf(temp, "%f", instr->getPan_L());
		TiXmlText pan_LValue(temp);
		pan_LNode.InsertEndChild(pan_LValue);
		instrumentNode.InsertEndChild(pan_LNode);

		TiXmlElement pan_RNode("pan_R");
		sprintf(temp, "%f", instr->getPan_R());
		TiXmlText pan_RValue(temp);
		pan_RNode.InsertEndChild(pan_RValue);
		instrumentNode.InsertEndChild(pan_RNode);

		// Delay level
		TiXmlElement delayLevelNode( "delayLevel" );
		sprintf( temp, "%f", instr->getDelayLevel() );
		TiXmlText delayLevelValue( temp );
		delayLevelNode.InsertEndChild( delayLevelValue );
		instrumentNode.InsertEndChild( delayLevelNode );

		TiXmlElement excludeNode( "exclude" );
		if (instr->excludeVectId.size() != 0) {
			for (uint i = 0; i <instr->excludeVectId.size(); i++) {
				LocalFileMng::writeXmlText( &excludeNode, "id" , instr->excludeVectId[ i ] );
			}
		}
		instrumentNode.InsertEndChild( excludeNode);

		instrumentListNode.InsertEndChild(instrumentNode);
	}

	songNode.InsertEndChild(instrumentListNode);





	// pattern list
	TiXmlElement patternListNode("patternList");


	uint nPatterns = song->getPatternList()->getSize();
	for (uint i = 0; i < nPatterns; i++) {
		Pattern *pat = song->getPatternList()->get(i);

		// pattern
		TiXmlElement patternNode("pattern");

		// pattern name
		TiXmlElement nameNode("name");
		TiXmlText nameValue(pat->getName());
		nameNode.InsertEndChild(nameValue);
		patternNode.InsertEndChild(nameNode);

		// pattern size
		TiXmlElement sizeNode( "size" );
		char tmp[10];
		sprintf(tmp, "%d", pat->getSize() );
		TiXmlText sizeValue( tmp );
		sizeNode.InsertEndChild( sizeValue );
		patternNode.InsertEndChild( sizeNode );

		// sequence list
		TiXmlElement sequenceListNode("sequenceList");

		uint nSequences = pat->getSequenceList()->getSize();
		for (uint j = 0; j < nSequences; j++) {
			Sequence *seq = pat->getSequenceList()->get(j);

			// Sequence
			TiXmlElement sequenceNode("sequence");

			TiXmlElement nameNode("name");
			TiXmlText nameValue(seq->getName());
			nameNode.InsertEndChild(nameValue);
			sequenceNode.InsertEndChild(nameNode);

			// Note List
			TiXmlElement noteListNode("noteList");

			uint nNotes = MAX_NOTES;
			for (uint y = 0; y < nNotes; y++) {
				Note *note = seq->noteList[y];
				if (note != NULL) {

					// note
					TiXmlElement noteNode("note");

					// position
					TiXmlElement positionNode("position");
					sprintf(temp, "%d", note->getPosition());
					TiXmlText positionValue(temp);
					positionNode.InsertEndChild(positionValue);
					noteNode.InsertEndChild(positionNode);

					// velocity
					TiXmlElement velocityNode("velocity");
					sprintf(temp, "%f", note->getVelocity());
					TiXmlText velocityValue(temp);
					velocityNode.InsertEndChild(velocityValue);
					noteNode.InsertEndChild(velocityNode);

					// pan_L
					TiXmlElement pan_LNode("pan_L");
					sprintf(temp, "%f", note->getPan_L());
					TiXmlText pan_LValue(temp);
					pan_LNode.InsertEndChild(pan_LValue);
					noteNode.InsertEndChild(pan_LNode);

					// pan_R
					TiXmlElement pan_RNode("pan_R");
					sprintf(temp, "%f", note->getPan_R());
					TiXmlText pan_RValue(temp);
					pan_RNode.InsertEndChild(pan_RValue);
					noteNode.InsertEndChild(pan_RNode);

					// pitch
					TiXmlElement pitchNode( "pitch" );
					sprintf( temp, "%f", note->getPitch() );
					TiXmlText pitchValue(temp);
					pitchNode.InsertEndChild( pitchValue );
					noteNode.InsertEndChild( pitchNode );

					// instrument
					TiXmlElement instrumentNode("instrument");
					string instrId = note->getInstrument()->getId();
					TiXmlText instrumentValue(instrId.c_str());
					instrumentNode.InsertEndChild(instrumentValue);
					noteNode.InsertEndChild(instrumentNode);

					noteListNode.InsertEndChild(noteNode);

				}
			}
			sequenceNode.InsertEndChild(noteListNode);
			sequenceListNode.InsertEndChild(sequenceNode);
		}
		patternNode.InsertEndChild(sequenceListNode);
		patternListNode.InsertEndChild(patternNode);
	}
	songNode.InsertEndChild(patternListNode);



	// pattern sequence
	TiXmlElement patternSequenceNode( "patternSequence" );
	uint nPatternSequence = song->getPatternSequence()->getSize();
	for ( uint i = 0; i < nPatternSequence; i++ ) {
		Pattern *pat = song->getPatternSequence()->get( i );

		string patName = "NULL";
		if (pat) {
			patName = pat->getName();
		}

		// pattern id
		TiXmlElement patternIDNode( "patternID" );
		TiXmlText patternIDValue( patName );
		patternIDNode.InsertEndChild( patternIDValue );
		patternSequenceNode.InsertEndChild( patternIDNode );
	}
	songNode.InsertEndChild( patternSequenceNode );


	doc.InsertEndChild(songNode);
        doc.SaveFile();

	song->setModified(false);
	song->setFilename(filename);
}





/**
 *
 */
string LocalFileMng::readXmlString( Object* obj, TiXmlNode* parent, string nodeName, string defaultValue ) {
	TiXmlNode* node;
	if ( ( node = parent->FirstChild( nodeName.c_str() ) ) ) {
		if ( node->FirstChild() ) {
			return node->FirstChild()->Value();
		}
		else {
			obj->warningLog( "Using default value in " + nodeName );
			return defaultValue;
		}
	}
	else {
		obj->warningLog( "Error reading XML node: " + nodeName + " node not found" );
		return defaultValue;
	}
}





/**
 *
 */
bool LocalFileMng::readXmlBool( Object* obj, TiXmlNode* parent, string nodeName, bool defaultValue ) {
	TiXmlNode* node;
	if ( ( node = parent->FirstChild( nodeName.c_str() ) ) ) {
		if ( node->FirstChild() ) {
			if ( string( node->FirstChild()->Value() ) == "true" ) {
				return true;
			}
			else {
				return false;
			}
		}
		else {
			obj->warningLog( "Using default value in " + nodeName );
			return defaultValue;
		}
	}
	else {
		obj->warningLog( "Error reading XML node: " + nodeName + " node not found" );
		return defaultValue;
	}
}





/**
 *
 */
void LocalFileMng::writeXmlText( TiXmlNode *parent, string name, string text ) {
	TiXmlElement versionNode( name );
	TiXmlText versionText( text );
	versionNode.InsertEndChild( versionText );
	parent->InsertEndChild( versionNode );
}




/**
 *
 */
void LocalFileMng::writeXmlText( TiXmlNode *parent, string name, float value ) {
	char value_char[80];
	sprintf( value_char, "%f", value );
	writeXmlText( parent, name, string(value_char) );
}




/**
 *
 */
void LocalFileMng::writeXmlText( TiXmlNode *parent, string name, int value ) {
	char value_char[80];
	sprintf( value_char, "%d", value );
	writeXmlText( parent, name, string(value_char) );
}
